import tempfile
from Network.NetworkTools import *
import DatabaseI

#wrapper for static functions. see http://code.activestate.com/recipes/52304/
class Callable:
    def __init__(self, anycallable):
        self.__call__ = anycallable


"""
@author W.Pasman 19aug09
This database contains is a very simple implementation of DatabaseI.
It is mainly a way to fetch "files"

Each of the files can be stored locally or remotely.
You use the get(name) to get an object. 
You then receive either 
* The same name if the file exists locally
* a string containing a path to a copy of that file on the database server.

EXAMPLE
1. Start the database server (see DatabaseServer.py). (This also starts the NetworkTools server)

2A. open file 
import Database.Database 
f=Database.Database.Database.get("somefile")
<< do something with file>>
f.close()


or easier, but only with python 2.6+:
import Database.Database 
with Database.Database.Database.get("somefile") as f:
	f.read()

"""

class Database(DatabaseI.DatabaseI):
	
	# copy file from database into tempfile and return tempfile
	# staticvars are to store static vars for our function
	def openCopyFromDatabase(filename, staticvars=[]):
		# first get proxy to databaseserver
		if len(staticvars)==0:
			# we don't have database proxy yet, get it.
			ad=NetworkTools.getInfo("databaseserver",True,10)
			if ad==-1:
				raise Exception("database is not available, can't open "+filename)
			proxy=xmlrpclib.ServerProxy(NetworkTools.makeFullName(ad))
			staticvars.append(proxy)
		else:
			proxy=staticvars[0]
		
		# get file from the server and put it in temp file.
		f=tempfile.NamedTemporaryFile()
		f.write(proxy.get(filename))
		f.seek(0)
		return f
			
	openCopyFromDatabase=Callable(openCopyFromDatabase)
	
	
	# @param filename is full path to a filename.
	# @returns open file handle, which is either the original, 
	# or a copy from the remote database if it does not exist under that name as original.
	#
	# NOTE if you work with a copy, writing to the file will not change the original!
	# We may have to fix this depending on how we think about this
	#
	# We avoid using "open" as a name for this as open is a built-in command that we need too.
	# After get you can use the returned path to open the file and access the contents.
	# @throws IOError if file does not exist, neither locally nor in database.
	# @precondition: DatabaseServer is already running. This is an object registered
	# as "database" on the NetworkTools server and fits the DatabaseServerI interface.
	# note that actual database access is necessary only when a file really has to be fetched.
	# this function will work fine as long as files are all found locally.
	def get(filename):
		try:
			return open(filename)
		except IOError: # not available, fetch a copy from the DatabaseServer
			print "fetching copy of "+filename+" from the server"
			return Database.openCopyFromDatabase(filename)
	
	get=Callable(get)



		
		
		