import Network
import Network.XMLRPCServer
import SUDScoreI
import xmlrpclib
import types
from socket import gethostname


"""
Simple SUD score analyser.
One instance per patient is allowed.
The "real" analyser should do speech recognition.
This one just expects the therapist to press buttons
The therapist interface then calls setSUD via an XMLRPC call to setSUD()
@author W.Pasman 17aug09


1+2. launch therapist widget (PhysStatusWidget and SUDSelWidget)

3. launch the SUDScore module:
import Patient.SUDScore
sud=Patient.SUDScore.SUDScore("patient1")

4A. Start the SUDSelWidget (seplease refer to that module)
4B simulate a therapist's click on a button:
import Network.NetworkTools 
ad=Network.NetworkTools.NetworkTools.getInfo("patient1.SUDScore",True,10)
import xmlrpclib 
proxy=xmlrpclib.ServerProxy(Network.NetworkTools.NetworkTools.makeFullName(ad))
proxy.setSUD(3)

this should result in 
1. message "setSUD arrived: 3" in the SUDScore module window
2. message "physstatuswidget received status update:3" in widget window

"""
class SUDScore(SUDScoreI.SUDScoreI):

	latest_sud=1
	patient =0 # patient id
	stop=False
	
	proxy=0 # proxy to the widget
	
	
	#@param pid is a patient id
	def __init__(self,pid):
		self.patient=pid
		
		# HACK we need more automatic port assignment.
		setsudserver=Network.XMLRPCServer.XMLRPCServer(self.patient+".SUDScore",(gethostname(),8003))
		setsudserver.register_function(self.setSUD, "setSUD")
		setsudserver.run()
		
		print "waiting for contact with the therapist's feedback widget"
		widgetnetinfo=Network.NetworkTools.NetworkTools.getInfo(self.patient+".PhysStatusWidget",True,10)
		if widgetnetinfo==-1:
			raise Exception("could not reach PhysStatusWidget")
		self.proxy=xmlrpclib.ServerProxy(Network.NetworkTools.NetworkTools.makeFullName(widgetnetinfo))

	
	
	def getSUD(self):
		return self.latest_sud
		
	def setSUD(self,new_sud):
		print ("setSUD arrived:"+str(new_sud))
		# do we need to use a lock here?? I guess this can't go wrong as we replace just one int?
		self.latest_sud=new_sud
		if type(self.proxy) == types.InstanceType: 
			# HACK due to bug, because proxy object (incorrectly?) overrides __eq__ and __uneq__ , therefore
			# self.proxy==0 gives <Fault 1: '<type \'exceptions.Exception\'>:method "__eq__" is not supported'>
			self.proxy.newStatus(new_sud)
		return 0 # prevent the dreaded xmlrpclib.Fault: <Fault 1: "<type 'exceptions.TypeError'>:cannot marshal None unless allow_none is enabled">

	def stop(self):
		self.stop=True
		

		